import os
import sys
import logging
from os import environ
import boto3
import threading
from botocore.exceptions import ClientError
from typing import Text, Optional
from storage_script.progress import ProgressPercentage


def download_file(model: Text, folder: Text):
    """Download a model to an S3 bucket

    :param model: model to download
    :param folder: S3 object name. 
    :return: True if file was downloaded, else False
    """
    s3_client = boto3.client("s3")
    try:
        response = s3_client.download_file(
            os.environ.get("AWS_S3_BUCKET_RASA_MODELS"),
            folder + "/" + model,
            model,
            Callback=ProgressPercentage(
                folder + "/" + model,
                s3_client,
                os.environ.get("AWS_S3_BUCKET_RASA_MODELS"),
            ),
        )
    except ClientError as e:
        logging.error(e)
        return False
    return True



import os
import sys
import logging
import tarfile
import argparse
from typing import List
from storage_script.upload import upload_file


def add_subparser(
    subparsers: argparse._SubParsersAction, parents: List[argparse.ArgumentParser]
):
    upload_parser = subparsers.add_parser(
        "upload",
        parents=parents,
        conflict_handler="resolve",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        help="Uploads RASA models to AWS S3",
    )
    upload_parser.set_defaults(func=upload)

    set_upload_arguments(upload_parser)


def set_upload_arguments(parser: argparse.ArgumentParser):
    parser.add_argument(
        "--model",
        "-m",
        required=True,
        type=str,
        help="Relative path to the Model (.tar.gz file)",
        default=None,
    )

    parser.add_argument(
        "--folder",
        "-f",
        required=False,
        type=str,
        help="Optional: Folder Name where the model needs to be saved in AWS S3",
        default="master",
    )


def upload(args: argparse.Namespace):

    if not os.path.exists(args.model):
        sys.exit(
            "Relative path {} does not exist! \n(for more information run the script with the --help option)".format(
                args.model
            )
        )
    elif not tarfile.is_tarfile(args.model):
        sys.exit(
            "The file {} to be uploaded is not a .tar.gz file. \nPlease upload a .tar.gz file.".format(
                args.model
            )
        )
    else:
        upload_file(args.model, args.folder)



import os
import sys
import logging
import argparse
import tarfile
from typing import List
from storage_script.download import download_file


def add_subparser(
    subparsers: argparse._SubParsersAction, parents: List[argparse.ArgumentParser]
):
    download_parser = subparsers.add_parser(
        "download",
        parents=parents,
        conflict_handler="resolve",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        help="Downloads RASA models from AWS S3",
    )
    download_parser.set_defaults(func=download)

    set_download_arguments(download_parser)


def set_download_arguments(parser: argparse.ArgumentParser):
    parser.add_argument(
        "--model",
        "-m",
        required=True,
        type=str,
        help="Model Name(.tar.gz file)",
        default=None,
    )

    parser.add_argument(
        "--folder",
        "-f",
        required=False,
        type=str,
        help="Optional: Folder Name from where the model needs to be downloaded",
        default="master",
    )


def download(args: argparse.Namespace):

    if not args.model.endswith(".tar.gz"):
        sys.exit(
            "The file {} to be downloaded is not a .tar.gz file. \nPlease enter a .tar.gz file.".format(
                args.model
            )
        )
    else:
        download_file(args.model, args.folder)



import os
import sys
import logging
from os import environ
import boto3
import threading
from botocore.exceptions import ClientError
from typing import Text, Optional
from storage_script.progress import ProgressPercentage


def upload_file(model: Text, folder: Text):
    """Upload a model to an S3 bucket

    :param model: model to upload
    :param folder: S3 object name. 
    :return: True if file was uploaded, else False
    """
    # Upload the file
    s3_client = boto3.client("s3")
    try:
        modelName = os.path.basename(model)
        response = s3_client.upload_file(
            model,
            os.environ.get("AWS_S3_BUCKET_RASA_MODELS"),
            folder + "/" + modelName,
            Callback=ProgressPercentage(model),
        )
    except ClientError as e:
        logging.error(e)
        return False
    return True



import os
import sys
import threading


class ProgressPercentage(object):
    def __init__(self, filename, client=None, bucket=None):
        self._filename = filename
        if client is None and bucket is None:
            self._size = float(os.path.getsize(filename))
        else:
            self._size = client.head_object(Bucket=bucket, Key=filename).get(
                "ContentLength"
            )
        self._seen_so_far = 0
        self._lock = threading.Lock()

    def __call__(self, bytes_amount):
        # To simplify, assume this is hooked up to a single filename
        with self._lock:
            self._seen_so_far += bytes_amount
            percentage = (self._seen_so_far / self._size) * 100
            sys.stdout.write(
                "\r%s  %s / %s  (%.2f%%)"
                % (self._filename, self._seen_so_far, self._size, percentage)
            )
            sys.stdout.flush()



import os
import sys
import logging
import argparse
from os import environ
import boto3
import threading
from botocore.exceptions import ClientError
from storage_script import __version__ as storage_script_version
from storage_script.cli import (
    upload,
    download,
)


def create_argument_parser() -> argparse.ArgumentParser:

    parser = argparse.ArgumentParser(
        description="Storage Script - Command line interface that uploads/downloads RASA models to/from AWS S3",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument(
        "--version",
        "-v",
        help="Package Version",
        action="version",
        version=storage_script_version,
    )

    parent_parser = argparse.ArgumentParser(add_help=False)

    parent_parsers = [parent_parser]

    subparsers = parser.add_subparsers(help="Rasa Storage Commands")

    upload.add_subparser(subparsers, parents=parent_parsers)
    download.add_subparser(subparsers, parents=parent_parsers)

    return parser


def main() -> None:
    checkEnvVariables()
    arg_parser = create_argument_parser()
    cmdline_arguments = arg_parser.parse_args()

    if hasattr(cmdline_arguments, "func"):
        cmdline_arguments.func(cmdline_arguments)
    elif hasattr(cmdline_arguments, "version"):
        print_version()
    else:
        arg_parser.print_help()
        exit(1)


def checkEnvVariables():
    for data in [
        "AWS_ACCESS_KEY_ID",
        "AWS_SECRET_ACCESS_KEY",
        "AWS_DEFAULT_REGION",
        "AWS_S3_BUCKET_RASA_MODELS",
    ]:
        env_var = os.environ.get(data)
        if not env_var:
            raise ValueError("Environment variable %s not set" % data)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['storage_script', 'storage_script.cli']

package_data = \
{'': ['*']}

install_requires = \
['boto3==1.13.22']

entry_points = \
{'console_scripts': ['rasa_storage = storage_script.__main__:main']}

setup_kwargs = {
    'name': 'storage-script',
    'version': '0.2.3',
    'description': 'Rasa Storage Script',
    'long_description': '# storage-script\n\nScript for uploading/downloading models to/from AWS S3 Cloud Storage\n\n- [Installation](#installation)\n- [Usage](#usage)\n\n## Environment Variables\n\nPlease make sure to add the following variables in your environment\n\n```\nexport AWS_ACCESS_KEY_ID=\nexport AWS_SECRET_ACCESS_KEY=\nexport AWS_DEFAULT_REGION=\nexport AWS_S3_BUCKET_RASA_MODELS=\nexport AWS_ENDPOINT_URL=\n```\n\n## Requirements\n\n* Local Python Installation\n* [Boto3](https://pypi.org/project/boto3/)\n\n## Installation\n```\npip install git+https://github.tools.sap/dsx/storage-script.git\n```\n\n## Usage\n\n```\nrasa_storage [-h] [--version] {upload,download} \n\n    positional arguments:\n    {upload,download}  Rasa Storage Commands\n        upload           Uploads RASA models to AWS S3\n        download         Downloads RASA models from AWS S3\n    optional arguments:\n    -h, --help         show this help message and exit\n    --version, -v      Package Version\n````\n\n### Upload\n\n```\nrasa_storage upload [-h] --model MODEL [--folder FOLDER]\n    -h, --help            show this help message and exit\n    --model MODEL, -m MODEL\n                            Relative path to the Model (.tar.gz file) (default:\n                            None)\n    \n    optional arguments:\n    --folder FOLDER, -f FOLDER\n                            Optional: Folder Name where the model needs to be\n                            saved in AWS S3 (default: master)\n````\n\n### Download\n\n```\nrasa_storage download [-h] --model MODEL [--folder FOLDER]\n\n    -h, --help            show this help message and exit\n    --model MODEL, -m MODEL\n                            Model Name(.tar.gz file) (default: None)\n    optional arguments:\n    --folder FOLDER, -f FOLDER\n                            Optional: Folder Name from where the model needs to be\n                        downloaded (default: master)\n```',
    'author': 'Venkatesh',
    'author_email': 'venkatesh.damodar.pai@sap.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.tools.sap/dsx/storage-script',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
